%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Purpose: Compute coefs of polynomial y(x) through points (xi,yi)
% Inputs:  
%   vx():   x-vector defining polynomial: len(vx)>1 required
%   vy():   y-vector defining polynomial: vy(i) = f(vx(i))
% Outputs: 
%   vc():   coefficients: y(x) = vc(i)*x^(i-1)
% Author: Martin Schubert
% Date last modified: 09.Apr.2017 by M. Schubert
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
function vc = f_PolyInit(vx,vy)
NoP = length(vx);
assert(nargin==2,'function requires 2 input vectors');
assert(NoP>1,'input vectors must have at least 2 points');
assert(length(vy)==NoP,'error: vx and vy must have same length');
A = zeros(NoP,NoP);
for row=1:NoP;
  for col=1:NoP;
    A(row,col) = vx(row)^(col-1);
  end;
end;
if size(vy,1)>1;
  vc = A\vy;  % compute coefficients with column vector vy
else
  vc = A\vy'; % the ' brings cy into the upright position
end;
vc=vc';