%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Module : f_adc
% Purpose: polynomial A/D Donverter (ADC) model with bounds
% Inputs : x:      vector of input values to be converted
%          alpha:  coefficients of DAC: alpha_k=alpha(k+(1))
%          bounds: =[bmin bmax]: output bounds, optional
%          noquant: no quantization if this parameter exists
% Outputs: n, same vector-length as x
% Author : Martin Schubert
% Date   : 23.Jul.2018
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
function n = f_adc(x,alpha,bounds,noquant)
n = 0;                % initialize
x_power_k=1;          % initialize x^k
for k=1:length(alpha); % evaluate polynomial
  n=n+alpha(k)*x_power_k;
  x_power_k = x_power_k.*x;
end;
if not(exist('noquant')); n = round(n); end; % round if not prohibited
if exist('bounds');                  % [ymin,ymax] boundaries available?
  n=max(bounds(1),min(bounds(2),n)); % clip output vector
end;