%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Module : f_dac
% Purpose: polynomial D/A Converter (DAC) model with bounds
% Inputs : n:      vector of input values to be converted
%          delta:  coefficients of DAC: delta_k=delta(k+(1))
%          bounds: =[bmin bmax]: output bounds optional
% Outputs: y, same vector-length as n
% Author : Martin Schubert
% Date   : 23.Jul.2018
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
function y = f_dac(n,delta,bounds)
y = 0;                   % initialize
if exist('delta');       % coefficient vector available?
  n_power_k=1;           % initialize n^k
  for k=1:length(delta); % evaluate polynomial
    y=y+delta(k)*n_power_k;
    n_power_k = n_power_k.*n;
  end;
end;
if exist('bounds');                  % [ymin,ymax] boundaries available?
  y=max(bounds(1),min(bounds(2),y)); % clip output vector
end;
