%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Module : f_quantize
% Purpose: model linear quantizer y=delta*round(x/delta)
% Inputs : x:      vector of input values to be converted
%          delta:  vector with 2 elements
%                  delta(1)=delta_0: offset.
%                  delta(2)=delta_1: step, when =0 no quantization.
%          bounds: =[bmin bmax]: output bounds, optional
% Outputs: y, same vector-length as x
% Author : Martin Schubert
% Date   : 19.Nov.2019
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
function y = f_quantize(x,delta,bounds)
if or(delta(2)==0, not(exist('delta')));
  y = x;
else
  y = delta(1) + delta(2)*round((x-delta(1))/delta(2));
end;
if nargin > 2;
  y=max(bounds(1),min(bounds(2),y)); % clip output vector
end;