%%%%%%%%%1%%%%%%%%%2%%%%%%%%%3%%%%%%%%%4%%%%%%%%%5%%%%%%%%%6%%%%%%%%%7%%%%%%%%%8
%
% Function: f_winCheb
%
% Purpose: compute taps of a Dolph-Chebychev window function
% 
% Output argument:
%   chebyWin : filter coefficients vector, with max(chebyWin)=1
%
% Input arguments:
%  required:
%  ---------
%   cTaps, int: >=1: number of taps
%               < 1: error
%
%  optional:
%  --------- 
%   DdB: Damping in dB, default is 80 
%
% Coded        : 17.Jul.2016 by Matthias Hausladen
% Last Revision: 07.Jun.2017 by Martin Schubert
%
% References:
%  http://practicalcryptography.com/miscellaneous/machine-learning/implementing-dolph-chebyshev-window/
% 
%%%%%%%%%%1%%%%%%%%%2%%%%%%%%%3%%%%%%%%%4%%%%%%%%%5%%%%%%%%%6%%%%%%%%%7%%%%%%%%%8
%
function chebyWin = f_winCheb(cTaps, DdB)
%
% check if input data makes sense
if (nargin < 2)
  DdB = 80;
end;
if (cTaps < 1)
  printf('\nError in f_WinCheby: less than 1 tap: %g',cTaps);
elseif (cTaps == 1)
   chebyWin = 1;
else
    iLoops = cTaps/2;
    iLoops = iLoops + 0.5;
    M = iLoops - 1;

    Sidelobedamp = 10^(DdB/20);
    x0 = cosh( (1/(cTaps-1)) * acosh(Sidelobedamp));

    for nn = 0:iLoops-1;
        n = nn - M;
        sumVal = 0;
        for i = 1:iLoops-1;
            x = x0 * cos( i * pi / cTaps);
            if( abs(x) <= 1 )
                sumVal = sumVal +  cos((cTaps-1)*acos(x)) * cos(2 * pi * i * n / cTaps);
            else
                sumVal = sumVal +  cosh((cTaps-1)*acosh(x)) * cos(2 * pi * i * n / cTaps);
            end;
        end;
        chebyWin(nn+1) = (Sidelobedamp + 2 * sumVal);
        chebyWin(cTaps-nn+1-1) = chebyWin(nn+1);
    end;

    chebyWin = chebyWin/max(chebyWin);   % Normalize to maximum of 1
    chebyWin = chebyWin';
end;