% tb_ada: Testbench for A/D and D/A converters
clear all; addpath('../../functions/');
%
% DAC specifications
%nda_c=[0  2       4       6        8   ]; % 5 n-inputs
%yda_c=[0  3.3*1/4 3.3*2/4 3.3*3/4  3.3 ]; % 5 y-outputs
nda_c=[0 1       2     3      4    5      6     7      8  ]; % 9 n-inputs
yda_c=nda_c*3.3/8;                                           % 9 n-outputs
%yda_c=[0 0.4125  0.825 1.2375 1.65 2.0625 2.475 2.8875 3.3]; % 9 n-outputs
% Typical measured DAC characteristics
%yda_c = [0 0.4140 0.8285 1.2430 1.6574 2.0718 2.4864 2.9009 3.3154];
% Untypical measured DAC characteristics needing 8. harmonic for SFDR
%yda_c = [0.0015 0.404 0.803 1.216 1.631 2.013 2.42 2.831 3.242];
    
bda   = [0.0 3.30];             % output clipping bounds for DAC
delta = f_PolyInit(nda_c,yda_c) % compute coefficints for DAC
%
% ADC specifications
xad_c = [0.5:7.5]*3.3/8;        % characteristic input data points  to ADC
nad_c = [0.5:7.5];              % characteristic output data from ADC
%nad_c(3) = 2.5001;              % add a non-linearity
bad   = [0     8];              % clipping bounds for ADC
alpha = f_PolyInit(xad_c,nad_c) % compute coefficints for ADC
%
% DC: I/O characteristics
NoS_x    = 1001;                % Number of x-Samples 
xmargin  = 0.5;                 % abscissa extension over x_c-bounds
xmin   = min(xad_c)-xmargin;    % left end of abscissa x
xmax   = max(xad_c)+xmargin;    % right end of abscissa x
xstep  = (xmax-xmin)/(NoS_x-1); % x step for measurement
x      = xmin:xstep:xmax;       % x for y(x) plot
%
% using the 2 functions: f_adc and f_dac
n_x    = f_adc(x,alpha,bad);           % quantized ADC output
y_x    = f_dac(n_x,delta,bda);         % DAC output from quantized n_x
nref_x = f_adc(x,alpha,bad,'noquant'); % unquantized ADC output nref_x
yref_x = f_dac(nref_x,delta,bda);      % DAC out from unquantized nref_x
eq_x   = y_x - yref_x;                 % quantization error
figure(61); subplot(321); plot(x,y_x,'k',x,yref_x,'b');
title('I/O Characteristics'); xlim([min(x) max(x)]);
xlabel('input voltage'); ylabel('analog & quantized'); grid on;
subplot(322); plot(x,eq_x,'r');  xlim([min(x) max(x)]);
title('errors'); xlabel('input voltage'); ylabel('error'); grid on;
%
% Transient: time domain
NoS_t  = 1001;        % Number of Samples on time axis
t      = 0:NoS_t-1;   % sampled time axis
NoW_t  = 19.0;        % Number of Waves ovr entire time axis
Fsig_t = NoW_t/NoS_t; % signal frequency relative to sampling rate
xamp_t = 1.65;        % amplitude of sinusoidal input signal
xoff_t = 1.65;        % offset of sinusoidal input signal
x_t    = xamp_t*sin(2*pi*Fsig_t*t) + xoff_t; % sinusoidal test signal
n_t    = f_adc(x_t,alpha,bad);           % quantized ADC output
y_t    = f_dac(n_t,delta,bda);           % DAC output from quantized n_t
nref_t = f_adc(x_t,alpha,bad,'noquant'); % analog ADC output as reference
yref_t = f_dac(nref_t,delta,bda);        % DAC output from  nref_t
eq_t   = y_t - yref_t;                   % quantization error
subplot(323); plot(t,y_t,'k',t,yref_t,'b'); xlim([1 1/Fsig_t]);
xlabel('time-domain sample'); ylabel('analog & quantized'); grid on;
subplot(324); plot(t,eq_t,'r'); xlim([1 1/Fsig_t]);
xlabel('time-domain sample'); ylabel('error'); grid on;

% apply window functions
SideLobeAttenuation_dB = 150; % Relative side-lobe atten., default: 100dB
win=ones(1,NoS_t);                            % rectangular window
%win=chebwin(NoS_t,SideLobeAttenuation_dB)';  % Matlab's chebwin
%win=f_winCheb(NoS_t,SideLobeAttenuation_dB); % selfmade Chebychev window
%win=blackman(NoS_t);                         % Blackman window
%win=blackmanharris(NoS_t);                   % Blackman-Harris window

% frequency domain
NoS_F  = NoS_t;                        % Number of frequency Samples
F      = [0:NoS_F-1]/NoS_F; % rel. Frequency axis, F=f/fs, fs:sampling rate
X_F    = f_dB(fft(x_t.*win)/NoS_F);    % spectral x_t=x(t)
Y_F    = f_dB(fft(y_t.*win)/NoS_F);    % spectral quantized  y_t=y(t)
Yref_F = f_dB(fft(yref_t.*win)/NoS_F); % spectral analog y(t)
Eq_F   = f_dB(fft(eq_t)/NoS_F,1e-8); % spectral error: y(t)-yref(t)
subplot(325); plot(F,Y_F,'k',F,Yref_F,'b'); xlim([0 0.5]);
xlabel('relative Frequency'); ylabel('analog & quantized [dB]'); grid on;
subplot(326); plot(F,Eq_F,'r'); xlim([0 0.5]);
xlabel('relative Frequency'); ylabel('error [dB]'); grid on;
%
%tb_characterize % call characterization script
